@@ -5,6 +5,7 @@ __all__ = (
 from typing import List, Tuple, Union
 from inspect import isawaitable
 from dataclasses import dataclass
+from contextlib import nullcontext
 
 from kivy.config import Config
 from kivy.properties import BooleanProperty, ListProperty, StringProperty, NumericProperty
@@ -126,11 +127,12 @@ class KXDraggableBehavior:
         drag_distance = self.drag_distance
 
         ox, oy = touch.opos
-        async for __ in ak.rest_of_touch_moves(self, touch):
-            dx = abs_(touch.x - ox)
-            dy = abs_(touch.y - oy)
-            if dy > drag_distance or dx > drag_distance:
-                return False
+        async with ak.watch_touch(self, touch) as is_touch_move:
+            while await is_touch_move():
+                dx = abs_(touch.x - ox)
+                dy = abs_(touch.y - oy)
+                if dy > drag_distance or dx > drag_distance:
+                    return False
         return True
 
     def drag_start_from_others_touch(self, touch_receiver: Widget, touch):
@@ -151,13 +153,9 @@ class KXDraggableBehavior:
         try:
             if touch_receiver is None:
                 original_pos_win = self.to_window(*self.pos)
-                if do_transform:
-                    touch.push()
-                    touch.apply_transform_2d(self.parent.to_widget)
-                offset_x = touch.ox - self.x
-                offset_y = touch.oy - self.y
-                if do_transform:
-                    touch.pop()
+                with temp_transform(touch, self.parent.to_widget) if do_transform else nullcontext():
+                    offset_x = touch.ox - self.x
+                    offset_y = touch.oy - self.y
             else:
                 offset_x = self.width * 0.5
                 offset_y = self.height * 0.5
@@ -197,9 +195,10 @@ class KXDraggableBehavior:
 
             # actual dragging process
             self.dispatch('on_drag_start', touch)
-            async for __ in ak.rest_of_touch_moves(self, touch):
-                self.x = touch.x - offset_x
-                self.y = touch.y - offset_y
+            async with ak.watch_touch(self, touch) as is_touch_move:
+                while await is_touch_move():
+                    self.x = touch.x - offset_x
+                    self.y = touch.y - offset_y
 
             # wait for other widgets to react to 'on_touch_up'
             await ak.sleep(-1)
@@ -231,12 +230,8 @@ class KXDraggableBehavior:
         # simulate 'on_touch_down'
         with temp_grab_current(touch):
             touch.grab_current = None
-            if do_transform:
-                touch.push()
-                touch.apply_transform_2d(self.parent.to_widget)
-            super().on_touch_down(touch)
-            if do_transform:
-                touch.pop()
+            with temp_transform(touch, self.parent.to_widget) if do_transform else nullcontext():
+                super().on_touch_down(touch)
 
         if not do_touch_up:
             return
@@ -245,8 +240,7 @@ class KXDraggableBehavior:
         # simulate 'on_touch_up'
         to_widget = self.to_widget if self.parent is None else self.parent.to_widget
         touch.grab_current = None
-        with temp_transform(touch):
-            touch.apply_transform_2d(to_widget)
+        with temp_transform(touch, to_widget):
             super().on_touch_up(touch)
 
         # simulate the grabbed one as well
@@ -256,8 +250,7 @@ class KXDraggableBehavior:
             if x is None:
                 continue
             touch.grab_current = x
-            with temp_transform(touch):
-                touch.apply_transform_2d(x.parent.to_widget)
+            with temp_transform(touch, x.parent.to_widget):
                 x.dispatch('on_touch_up', touch)
 
         touch.grab_current = None
@@ -281,7 +274,7 @@ class KXDraggableBehavior:
     async def on_drag_fail(self, touch):
         ctx = self._drag_ctx
         await ak.animate(
-            self, d=.1,
+            self, duration=.1,
             x=ctx.original_pos_win[0],
             y=ctx.original_pos_win[1],
         )
@@ -379,12 +372,11 @@ class KXReorderableBehavior:
             if drag_cls is not None:
                 touch_ud[ud_key] = None
                 if drag_cls in self.drag_classes:
-                    # Watches 'is_being_dragged' as well, so that the
-                    # '_watch_touch()' will be automatically cancelled when
-                    # the drag is cancelled.
+                    # Start watching the touch. Use ``ak.or_()`` so that ``_watch_touch()`` will be automatically
+                    # cancelled when the drag is cancelled.
                     ak.start(ak.or_(
                         self._watch_touch(touch),
-                        ak.event(touch.ud['kivyx_draggable'], 'is_being_dragged'),
+                        ak.event(touch.ud['kivyx_draggable'], 'on_drag_end'),
                     ))
         return super().on_touch_move(touch)
 
@@ -405,16 +397,17 @@ class KXReorderableBehavior:
                 touch_ud['kivyx_draggable'].drag_context.original_location,
                 ignore_parent=True)
             add_widget(spacer)
-            async for __ in ak.rest_of_touch_moves(self, touch):
-                x, y = touch.pos
-                if collide_point(x, y):
-                    new_idx = get_drop_insertion_index_move(x, y, spacer)
-                    if new_idx is not None:
-                        remove_widget(spacer)
-                        add_widget(spacer, index=new_idx)
-                else:
-                    del touch_ud[self.__ud_key]
-                    return
+            async with ak.watch_touch(self, touch) as is_touch_move:
+                while await is_touch_move():
+                    x, y = touch.pos
+                    if collide_point(x, y):
+                        new_idx = get_drop_insertion_index_move(x, y, spacer)
+                        if new_idx is not None:
+                            remove_widget(spacer)
+                            add_widget(spacer, index=new_idx)
+                    else:
+                        del touch_ud[self.__ud_key]
+                        return
             if 'kivyx_droppable' not in touch_ud:
                 touch_ud['kivyx_droppable'] = self
                 touch_ud['kivyx_droppable_index'] = self.children.index(spacer)
