@@ -4,26 +4,37 @@ __all__ = (
 )
 
 from weakref import ref
-from contextlib import contextmanager
 from copy import deepcopy
 
 
-@contextmanager
-def temp_transform(touch):
-    touch.push()
-    try:
-        yield
-    finally:
-        touch.pop()
+class temp_transform:
+    __slots__ = ('_touch', '_func')
+
+    def __init__(self, touch, func):
+        self._touch = touch
+        self._func = func
+
+    def __enter__(self):
+        t = self._touch
+        t.push()
+        t.apply_transform_2d(self._func)
+
+    def __exit__(self, *args):
+        self._touch.pop()
 
 
-@contextmanager
-def temp_grab_current(touch):
-    original = touch.grab_current
-    try:
-        yield
-    finally:
-        touch.grab_current = original
+class temp_grab_current:
+    __slots__ = ('_touch', '_original', )
+
+    def __init__(self, touch):
+        self._touch = touch
+        self._original = touch.grab_current
+
+    def __enter__(self):
+        pass
+
+    def __exit__(self, *args):
+        self._touch.grab_current = self._original
 
 
 _shallow_copyable_property_names = (
